from dios import *
import pytest

s1 = pd.Series(range(10), index=range(10))
s2 = pd.Series(range(5, 10), index=range(5, 10))
s3 = pd.Series(range(1, 30, 2), index=range(1, 30, 2))
s4 = pd.Series(np.linspace(7, 13, 9), index=range(3, 12))
s1.name, s2.name, s3.name, s4.name = 'a', 'b', 'c', 'd'
d1 = DictOfSeries(data=dict(a=s1.copy(), b=s2.copy(), c=s3.copy(), d=s4.copy()))


@pytest.mark.parametrize(('idxer', 'exp'), [('a', s1), ('c', s3)])
def test__getitem_single(idxer, exp):
    a = d1[idxer]
    b = d1.loc[:, idxer]
    assert isinstance(a, pd.Series)
    assert isinstance(b, pd.Series)
    assert (a == exp).all()
    assert (b == exp).all()


@pytest.mark.parametrize(('idxer', 'exp'), [((1, 'a'), s1), ((3, 'c'), s3)])
def test__getitem_scalar_loc(idxer, exp):
    a = d1.loc[idxer]
    assert is_scalar(a)
    assert a == exp.loc[idxer[0]]


@pytest.mark.parametrize(('idxer', 'exp'), [(0, s1), (1, s2), (2, s3), (3, s4),
                                            (-1, s4), (-2, s3), (-3, s2), (-4, s1)])
def test__getitem_single_iloc(idxer, exp):
    a = d1.iloc[:, idxer]
    assert isinstance(a, pd.Series)
    assert (a == exp).all()


@pytest.mark.parametrize(('idxer', 'exp'), [((1, 0), s1), ((3, -2), s3), ((-1, -1), s4)])
def test__getitem_scalar_iloc(idxer, exp):
    a = d1.iloc[idxer]
    assert is_scalar(a)
    assert a == exp.iloc[idxer[0]]


@pytest.mark.parametrize('idxer', ['x', '2', 1, None, ])
def test__getitem_single_fail(idxer):
    with pytest.raises(KeyError):
        a = d1[idxer]
        print(idxer, a)


@pytest.mark.parametrize('idxer', ['x', '2', 1, None, ])
def test__getitem_single_loc_fail(idxer):
    with pytest.raises(KeyError):
        a = d1.loc[:, idxer]


@pytest.mark.parametrize('idxer', [-5, 99, 'a', '2', None, ])
def test__getitem_single_iloc_fail(idxer):
    with pytest.raises(KeyError):
        a = d1.iloc[:, idxer]


BLIST = [True, False, False, True]

LISTIDXER = [['a'], ['a', 'c'], pd.Series(['a', 'c'])]
BOOLIDXER = [pd.Series(BLIST), d1.copy() > 10]
SLICEIDXER = [slice(None), slice(-3, -1), slice(-1, 3), slice(None, None, 3)]
MULTIIDXER = []  # [d1 > 9, d1 != d1, d1 == d1]
EMPTYIDEXER = [[], pd.Series(), slice(3, 3), slice(3, -1), DictOfSeries()]

INDEXERS = LISTIDXER + BOOLIDXER + SLICEIDXER + MULTIIDXER + EMPTYIDEXER


@pytest.mark.parametrize('idxer', INDEXERS)
def test__getitem__(idxer):
    d = d1[idxer]
    assert isinstance(d, DictOfSeries)


FAIL_INDEXERS = [['z'], ['a', 'z'], pd.Series(['a', 'z']), BLIST, DictOfSeries(dict(a=[1, 2, 3]))]


@pytest.mark.parametrize('idxer', FAIL_INDEXERS)
def test__getitem__fail(idxer):
    with pytest.raises((ValueError, KeyError)):
        d1[idxer]

